--[[

$Revision: 54730 $

(C) Copyright 2007 Bethink (bethink@naef.com)

This file is part of CrossRealm.

CrossRealm is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CrossRealm is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CrossRealm. If not, see <http://www.gnu.org/licenses/>.

]]


-- Acquire libraries and declare add-on

local L = AceLibrary("AceLocale-2.2"):new("CrossRealm")
local waterfall = AceLibrary("Waterfall-1.0")
CrossRealm = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0")


-- Constants

local qualityMap = {
	[0] = L["poor"],
	[1] = L["common"],
	[2] = L["uncommon"],
	[3] = L["rare"],
	[4] = L["epic"]
}

local inverseQualityMap = { }
for key, value in pairs(qualityMap) do
	inverseQualityMap[value] = key
end

local opts = {
	type = "group",
	args = {
		minProfit = {
			type = "text",
			name = "minProfit",
			desc = L["minProfit"],
			usage = L["minProfitUsage"],
			get = "GetMinProfit",
			set = "SetMinProfit",
			validate = "ValidateMinProfit"
		},
		minProfitPct = {
			type = "text",
			name = "minProfitPct",
			desc = L["minProfitPct"],
			usage = L["minProfitPctUsage"],
			get = "GetMinProfitPct",
			set = "SetMinProfitPct",
			validate = "ValidateMinProfitPct"
		},
		minQuality = {
			type = "text",
			name = "minQuality",
			desc = L["minQuality"],
			get = "GetMinQuality",
			set = "SetMinQuality",
			validate = { L["poor"], L["common"], L["uncommon"], L["rare"], L["epic"] }
		},
		waterfall = {
			type = "execute",
			name = "waterfall",
			desc = L["waterfall"],
			func = "Waterfall"
		},
		wowecon = {
			type = "execute",
			name = "wowecon",
			desc = L["wowecon"],
			func = "ReportWowecon"
		}
	}
}

function CrossRealm:OnInitialize()
	self:RegisterChatCommand(L["slashCommands"], opts)
	waterfall:Register("CrossRealm", "aceOptions", opts)
	self:RegisterDB("CrossRealmDB", "CrossRealmDBPerCharacter")
	self:RegisterDefaults("profile", {
		minProfit = 10000, -- 1 gold
		minProfitPct = 20,
		minQuality = 3
	})
end

function CrossRealm:OnEnable ()
end

function CrossRealm:OnDisable ()
end

function CrossRealm:GetMinProfit ()
	return self.db.profile.minProfit
end

function CrossRealm:SetMinProfit (minProfit)
	self.db.profile.minProfit = tonumber(minProfit)
end

function CrossRealm:ValidateMinProfit (minProfit)
	local number = tonumber(minProfit)
	return number ~= nil and number >= 0
end

function CrossRealm:GetMinProfitPct ()
	return self.db.profile.minProfitPct
end

function CrossRealm:SetMinProfitPct (minProfitPct)
	self.db.profile.minProfitPct = tonumber(minProfitPct)
end

function CrossRealm:ValidateMinProfitPct (minProfitPct)
	local number = tonumber(minProfitPct)
	return number ~= nil and number >= 0 and number <= 100
end

function CrossRealm:GetMinQuality ()
	return qualityMap[self.db.profile.minQuality]
end

function CrossRealm:SetMinQuality (minQuality)
	self.db.profile.minQuality = inverseQualityMap[minQuality]
end

function CrossRealm:Waterfall ()
	waterfall:Open("CrossRealm")
end

function CrossRealm:ReportWowecon ()
	-- Check is WOWEcon_PriceMod is available
	if not IsAddOnLoaded("WOWEcon_PriceMod") then
		DEFAULT_CHAT_FRAME:AddMessage(L["WOWEcon_PriceModNotLoaded"])
		return
	end

	-- Load auctioneer as needed
	if not IsAddOnLoaded("Auctioneer") then
		LoadAddOn("Auctioneer")
	end

	-- Get a snapshot and process
	local auctions = Auctioneer.SnapshotDB.Query(nil, nil, CrossRealm.AuctioneerFilter, nil)
	for _, auction in pairs(auctions) do
		local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
		local itemLink = Auctioneer.ItemDB.GetItemLink(itemKey)
		local woweconPrice = Wowecon.API.GetAuctionPrice_ByLink(itemLink, Wowecon.API.GLOBAL_PRICE)
		if woweconPrice then
			local buyoutPrice = auction.buyoutPrice / auction.count
			local profit = woweconPrice - buyoutPrice
			local profitPct = profit / buyoutPrice * 100
			if profit >= self.db.profile.minProfit and profitPct >= self.db.profile.minProfitPct then
				DEFAULT_CHAT_FRAME:AddMessage(format(
					L["ReportLine"],
					itemLink,
					auction.count,
					buyoutPrice / 10000,
					woweconPrice / 10000,
					profit / 10000,
					profitPct))
			end
		end
	end
end

function CrossRealm.AuctioneerFilter (auction, filterArgs)
	-- Check quality
	local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
	local itemInfo = Auctioneer.ItemDB.GetItemInfo(itemKey)
	if itemInfo.quality < CrossRealm.db.profile.minQuality then
		return false
	end

	-- Check for buyout auction
	if auction.buyoutPrice == 0 then
		return false
	end

	-- Passed
	return true
end




